@@ -16,7 +16,6 @@ use vars qw(
     @ISA @EXPORT @EXPORT_OK 
     $Verbose $Switches $Debug
     $verbose $switches $debug
-    $Curtest
     $Columns 
     $Timer
     $ML $Last_ML_Print
@@ -35,11 +34,11 @@ Test::Harness - Run Perl standard test scripts with statistics
 
 =head1 VERSION
 
-Version 2.57_02
+Version 2.57_03
 
 =cut
 
-$VERSION = "2.57_02";
+$VERSION = "2.57_03";
 
 # Backwards compatibility for exportable variable names.
 *verbose  = *Verbose;
@@ -63,7 +62,7 @@ sub strap { return $Strap };
 
 @ISA = ('Exporter');
 @EXPORT    = qw(&runtests);
-@EXPORT_OK = qw($verbose $switches);
+@EXPORT_OK = qw(&execute_tests $verbose $switches);
 
 $Verbose  = $ENV{HARNESS_VERBOSE} || 0;
 $Debug    = $ENV{HARNESS_DEBUG} || 0;
@@ -190,15 +189,11 @@ abbreviated (ie. 15-20 to indicate that tests 15, 16, 17, 18, 19 and
 =back
 
 
-=head2 Functions
+=head1 FUNCTIONS
 
-Test::Harness currently only has one function, here it is.
+The following functions are available.
 
-=over 4
-
-=item B<runtests>
-
-  my $allok = runtests(@test_files);
+=head2 runtests( @test_files )
 
 This runs all the given I<@test_files> and divines whether they passed
 or failed based on their output to STDOUT (details above).  It prints
@@ -215,8 +210,8 @@ sub runtests {
 
     local ($\, $,);
 
-    my($tot, $failedtests) = _run_all_tests(@tests);
-    _show_results($tot, $failedtests);
+    my($tot, $failedtests) = execute_tests(tests => \@tests);
+    print get_results($tot, $failedtests);
 
     my $ok = _all_ok($tot);
 
@@ -226,15 +221,8 @@ sub runtests {
     return $ok;
 }
 
-=begin _private
-
-=item B<_all_ok>
-
-  my $ok = _all_ok(\%tot);
-
-Tells you if this test run is overall successful or not.
-
-=cut
+# my $ok = _all_ok(\%tot);
+# Tells you if this test run is overall successful or not.
 
 sub _all_ok {
     my($tot) = shift;
@@ -242,30 +230,30 @@ sub _all_ok {
     return $tot->{bad} == 0 && ($tot->{max} || $tot->{skipped}) ? 1 : 0;
 }
 
-=item B<_globdir>
+# Returns all the files in a directory.  This is shorthand for backwards
+# compatibility on systems where C<glob()> doesn't work right.
 
-  my @files = _globdir $dir;
+sub _globdir {
+    local *DIRH;
 
-Returns all the files in a directory.  This is shorthand for backwards
-compatibility on systems where C<glob()> doesn't work right.
-
-=cut
-
-sub _globdir { 
-    opendir DIRH, shift; 
-    my @f = readdir DIRH; 
-    closedir DIRH; 
+    opendir DIRH, shift;
+    my @f = readdir DIRH;
+    closedir DIRH;
 
     return @f;
 }
 
-=item B<_run_all_tests>
+=head2 execute_tests( tests => \@test_files, out => \*FH )
 
-  my($total, $failed) = _run_all_tests(@test_files);
+Runs all the given C<@test_files> (just like C<runtests()>) but
+doesn't generate the final report.  During testing, progress
+information will be written to the currently selected output
+filehandle (usually C<STDOUT>), or to the filehandle given by the
+C<out> parameter.  The I<out> is optional.
 
-Runs all the given C<@test_files> (as C<runtests()>) but does it
-quietly (no report).  $total is a hash ref summary of all the tests
-run.  Its keys and values are this:
+Returns a list of two values, C<$total> and C<$failed>, describing the
+results.  C<$total> is a hash ref summary of all the tests run.  Its
+keys and values are this:
 
     bonus           Number of individual todo tests unexpectedly passed
     max             Number of individual tests ran
@@ -282,7 +270,7 @@ run.  Its keys and values are this:
 If C<< $total->{bad} == 0 >> and C<< $total->{max} > 0 >>, you've
 got a successful test.
 
-$failed is a hash ref of all the test scripts which failed.  Each key
+C<$failed> is a hash ref of all the test scripts that failed.  Each key
 is the name of a test script, each value is another hash representing
 how that script failed.  Its keys are these:
 
@@ -296,22 +284,16 @@ how that script failed.  Its keys are these:
 
 C<$failed> should be empty if everything passed.
 
-B<NOTE> Currently this function is still noisy.  I'm working on it.
-
 =cut
 
-# Turns on autoflush for the handle passed
-sub _autoflush {
-    my $flushy_fh = shift;
-    my $old_fh = select $flushy_fh;
-    $| = 1;
-    select $old_fh;
-}
+sub execute_tests {
+    my %args = @_;
+    my @tests = @{$args{tests}};
+    my $out = $args{out} || select();
 
-sub _run_all_tests {
-    my @tests = @_;
-
-    _autoflush(\*STDOUT);
+    # We allow filehandles that are symbolic refs
+    no strict 'refs';
+    _autoflush($out);
     _autoflush(\*STDERR);
 
     my(%failedtests);
@@ -341,13 +323,13 @@ sub _run_all_tests {
         my($leader, $ml) = _mk_leader($tfile, $width);
         local $ML = $ml;
 
-        print $leader;
+        print $out $leader;
 
         $tot{files}++;
 
         $Strap->{_seen_header} = 0;
         if ( $Test::Harness::Debug ) {
-            print "# Running: ", $Strap->_command_line($tfile), "\n";
+            print $out "# Running: ", $Strap->_command_line($tfile), "\n";
         }
         my $test_start_time = $Timer ? time : 0;
         my %results = $Strap->analyze_file($tfile) or
@@ -397,17 +379,17 @@ sub _run_all_tests {
                     if $test{skipped};
                 push(@msg, "$test{bonus}/$test{max} unexpectedly succeeded")
                     if $test{bonus};
-                print "$test{ml}ok$elapsed\n        ".join(', ', @msg)."\n";
+                print $out "$test{ml}ok$elapsed\n        ".join(', ', @msg)."\n";
             }
             elsif ( $test{max} ) {
-                print "$test{ml}ok$elapsed\n";
+                print $out "$test{ml}ok$elapsed\n";
             }
             elsif ( defined $test{skip_all} and length $test{skip_all} ) {
-                print "skipped\n        all skipped: $test{skip_all}\n";
+                print $out "skipped\n        all skipped: $test{skip_all}\n";
                 $tot{skipped}++;
             }
             else {
-                print "skipped\n        all skipped: no reason given\n";
+                print $out "skipped\n        all skipped: no reason given\n";
                 $tot{skipped}++;
             }
             $tot{good}++;
@@ -435,7 +417,7 @@ sub _run_all_tests {
                 if (@{$test{failed}} and $test{max}) {
                     my ($txt, $canon) = _canonfailed($test{max},$test{skipped},
                                                     @{$test{failed}});
-                    print "$test{ml}$txt";
+                    print $out "$test{ml}$txt";
                     $failedtests{$tfile} = { canon   => $canon,
                                              max     => $test{max},
                                              failed  => scalar @{$test{failed}},
@@ -446,7 +428,7 @@ sub _run_all_tests {
                                            };
                 }
                 else {
-                    print "Don't know which tests failed: got $test{ok} ok, ".
+                    print $out "Don't know which tests failed: got $test{ok} ok, ".
                           "expected $test{max}\n";
                     $failedtests{$tfile} = { canon   => '??',
                                              max     => $test{max},
@@ -460,7 +442,7 @@ sub _run_all_tests {
                 $tot{bad}++;
             }
             else {
-                print "FAILED before any test output arrived\n";
+                print $out "FAILED before any test output arrived\n";
                 $tot{bad}++;
                 $failedtests{$tfile} = { canon       => '??',
                                          max         => '??',
@@ -480,7 +462,7 @@ sub _run_all_tests {
                 @f{@new_dir_files} = (1) x @new_dir_files;
                 delete @f{@dir_files};
                 my @f = sort keys %f;
-                print "LEAKED FILES: @f\n";
+                print $out "LEAKED FILES: @f\n";
                 @dir_files = @new_dir_files;
             }
         }
@@ -492,9 +474,17 @@ sub _run_all_tests {
     return(\%tot, \%failedtests);
 }
 
-=item B<_mk_leader>
+# Turns on autoflush for the handle passed
+sub _autoflush {
+    my $flushy_fh = shift;
+    my $old_fh = select $flushy_fh;
+    $| = 1;
+    select $old_fh;
+}
 
-  my($leader, $ml) = _mk_leader($test_file, $width);
+=for private _mk_leader
+
+    my($leader, $ml) = _mk_leader($test_file, $width);
 
 Generates the 't/foo........' leader for the given C<$test_file> as well
 as a similar version which will overwrite the current line (by use of
@@ -523,7 +513,7 @@ sub _mk_leader {
     return($leader, $ml);
 }
 
-=item B<_leader_width>
+=for private _leader_width
 
   my($width) = _leader_width(@test_files);
 
@@ -546,15 +536,15 @@ sub _leader_width {
     return $maxlen + 3 - $maxsuflen;
 }
 
-
-sub _show_results {
+sub get_results {
     my($tot, $failedtests) = @_;
+    my $out = '';
 
     my $pct;
     my $bonusmsg = _bonusmsg($tot);
 
     if (_all_ok($tot)) {
-        print "All tests successful$bonusmsg.\n";
+        $out .= "All tests successful$bonusmsg.\n";
     }
     elsif (!$tot->{tests}){
         die "FAILED--no tests were run for some reason.\n";
@@ -571,23 +561,34 @@ sub _show_results {
                               $tot->{max} - $tot->{ok}, $tot->{max}, 
                               $percent_ok;
 
-        my($fmt_top, $fmt) = _create_fmts($failedtests);
+        my($fmt_top, $fmt1, $fmt2) = _create_fmts($failedtests);
 
         # Now write to formats
         for my $script (sort keys %$failedtests) {
-          $Curtest = $failedtests->{$script};
-          write;
+            my $Curtest = $failedtests->{$script};
+            $out .= swrite( $fmt_top );
+            $out .= swrite( $fmt1, @{ $Curtest }{qw(name estat wstat max failed percent canon)} );
+            $out .= swrite( $fmt2, $Curtest->{canon} );
         }
         if ($tot->{bad}) {
             $bonusmsg =~ s/^,\s*//;
-            print "$bonusmsg.\n" if $bonusmsg;
-            die "Failed $tot->{bad}/$tot->{tests} test scripts, $pct% okay.".
-                "$subpct\n";
+            $out .= "$bonusmsg.\n" if $bonusmsg;
+            $out .= "Failed $tot->{bad}/$tot->{tests} test scripts, $pct% okay.$subpct\n";
         }
     }
 
-    printf("Files=%d, Tests=%d, %s\n",
+    $out .= sprintf("Files=%d, Tests=%d, %s\n",
            $tot->{files}, $tot->{max}, timestr($tot->{bench}, 'nop'));
+    return $out;
+}
+
+sub swrite {
+    my $format = shift;
+    $^A = '';
+    formline($format,@_);
+    my $out = $^A;
+    $^A = '';
+    return $out;
 }
 
 
@@ -758,33 +759,19 @@ sub _create_fmts {
         }
     }
 
-    my $fmt_top = "format STDOUT_TOP =\n"
-                  . sprintf("%-${max_namelen}s", $failed_str)
+    my $fmt_top =   sprintf("%-${max_namelen}s", $failed_str)
                   . $middle_str
                   . $list_str . "\n"
                   . "-" x $Columns
-                  . "\n.\n";
+                  . "\n";
 
-    my $fmt = "format STDOUT =\n"
-              . "@" . "<" x ($max_namelen - 1)
+    my $fmt1 =  "@" . "<" x ($max_namelen - 1)
               . "  @>> @>>>> @>>>> @>>> ^##.##%  "
-              . "^" . "<" x ($list_len - 1) . "\n"
-              . '{ $Curtest->{name}, $Curtest->{estat},'
-              . '  $Curtest->{wstat}, $Curtest->{max},'
-              . '  $Curtest->{failed}, $Curtest->{percent},'
-              . '  $Curtest->{canon}'
-              . "\n}\n"
-              . "~~" . " " x ($Columns - $list_len - 2) . "^"
-              . "<" x ($list_len - 1) . "\n"
-              . '$Curtest->{canon}'
-              . "\n.\n";
+              . "^" . "<" x ($list_len - 1) . "\n";
+    my $fmt2 =  "~~" . " " x ($Columns - $list_len - 2) . "^"
+              . "<" x ($list_len - 1) . "\n";
 
-    eval $fmt_top;
-    die $@ if $@;
-    eval $fmt;
-    die $@ if $@;
-
-    return($fmt_top, $fmt);
+    return($fmt_top, $fmt1, $fmt2);
 }
 
 sub _canonfailed ($$@) {
@@ -836,16 +823,9 @@ sub _canonfailed ($$@) {
     }
     push @result, "\n";
     my $txt = join "", @result;
-    ($txt, $canon);
+    return ($txt, $canon);
 }
 
-=end _private
-
-=back
-
-=cut
-
-
 1;
 __END__
 
@@ -854,7 +834,8 @@ __END__
 
 C<&runtests> is exported by Test::Harness by default.
 
-C<$verbose>, C<$switches> and C<$debug> are exported upon request.
+C<&execute_tests>, C<$verbose>, C<$switches> and C<$debug> are
+exported upon request.
 
 =head1 DIAGNOSTICS
 
@@ -1081,6 +1062,11 @@ L<http://search.cpan.org/dist/Test-Harness>
 
 =back
 
+=head1 SOURCE CODE
+
+The source code repository for Test::Harness is at
+L<http://svn.perl.org/modules/Test-Harness>.
+
 =head1 AUTHORS
 
 Either Tim Bunce or Andreas Koenig, we don't know. What we know for
